//  ExtImage/ExtImage.h
//  A part of ViewIt image viewer
//  Copyright (c) 2003 Zdzislaw Losvik. All rights reserved.

#import <Cocoa/Cocoa.h>

extern NSString *ExtImageAttributeScaleKey;
extern NSString *ExtImageAttributeOriginXKey;
extern NSString *ExtImageAttributeOriginYKey;
extern NSString *ExtImageAttributesFlipHorizontalKey;
extern NSString *ExtImageAttributesFlipVerticalKey;
extern NSString *ExtImageAttributesRotateCWKey;

// snapshot extended data
extern NSString *ExtImageAttributeInterpolationKey; // NSNumber, int

extern NSString *ExtImageAttributeFitUpscaleKey; // NSNumber, bool

extern NSString *ExtImageAttributeFitModeKey; // NSString, Fit...Mode value
extern NSString *ExtImageAttributeFitFrameMode;
extern NSString *ExtImageAttributeFitWidthMode;
extern NSString *ExtImageAttributeFitSizeWidthKey; // NSNumber, float
extern NSString *ExtImageAttributeFitSizeHeightKey; // NSNumber, float

#ifdef USE_VI_COLOR_SYNC
extern NSString *ExtImageProfileKey; // NSData, icc profile data
#endif

enum {
  ExtImageTransformDefault = 0,
  
  ExtImageFlipHorizontalMask = 1 << 3,
  ExtImageFlipVerticalMask = 1 << 2,
  ExtImageRotationMask = 3
};

typedef short ExtImageAttributes;

enum {
  ExtImageRotationNone = 0,
  ExtImageRotation90 = 1,
  ExtImageRotation180 = 2,
  ExtImageRotation270 = 3,
};
typedef short ExtImageRotation;

@interface ExtImage : NSImage {
  
  // internal presentation data
  ExtImageAttributes	_extAttributes;
  NSPoint				_extOrigin;
  float				_extScale;
  NSAffineTransform	*_extTransform;
  NSData				*_extICCProfile;
  NSData				*_extOriginalICCProfile;
  
  // internal data
  BOOL				_extRebuildTransform;
  
  // data information
  NSArray				*_imageReps; // set if bitmap image with more than one page
  NSPDFImageRep		*_pdfImageRep; // set if data is PDF with more than one page
  NSInteger					_pageCount;
  NSInteger					_currentPage;
  BOOL				_mayHaveNoBackground;
  
  ExtImageAttributes	_extSnapShotAttributes;
  float				_extSnapShotScale;
  NSImage				*_extSnapShotImage;
  BOOL				_extAutoSnapShot;
  NSLock				*_extSnapshotCreationInProgressLock;
  NSImageInterpolation _extSnapShotInterpolation, _extInterpolation;
  NSData				*_extSnapShotICCProfile;
  
  BOOL				_allowSnapshots;
}

+ (float)fitSizeFactorForObjectSize:(NSSize)objectSize inFrameSize:(NSSize)containerSize;

+ (float)fitWidthFactorForObjectSize:(NSSize)objectSize inFrameSize:(NSSize)containerSize;

+ (ExtImageAttributes)simplifyAttributes:(ExtImageAttributes)attirbutes;

- (void)fitFrameSize:(NSSize)containerSize centered:(BOOL)centered;

- (void)fitFrame:(NSRect)containerRect centered:(BOOL)centered;

- (instancetype)initWithData:(NSData *)data;

- (void)setPageNumber:(NSInteger)pageNumber;

- (NSAffineTransform *)extTransform;

- (NSAffineTransform *)extInverseTransform;

- (void)setExtAttributes:(ExtImageAttributes)newAttirbutes;

- (ExtImageAttributes)extAttributes;

- (void)setExtOrigin:(NSPoint)newOrigin;

- (void)translateExtOriginXBy:(float)dx YBy:(float)dy;

- (NSPoint)extOrigin;

- (void)setExtScale:(float)newScale;

- (NSSize)extSizeNoScaling;

- (float)extScale;

- (void)extRotateCW:(BOOL)clockwise;

- (NSSize)extSize;

- (void) drawWithOperation:(NSCompositingOperation)operation fraction:(float)fraction;

//- (void)extDrawInRect:(NSRect)destRect fromRect:(NSRect)srcRect operation:(NSCompositingOperation)operation fraction:(float)fraction;

- (void)resetExtData;

- (BOOL)mayHaveNoBackground;

- (NSInteger)pageCount;

- (void)makeSnapShot;

- (void)removeSnapShot;

- (void)setAutoSnapShot:(BOOL)autoSnapShot;

- (BOOL)isAutoSnapShot;

- (void)setAllowSnapshots:(BOOL)allowSnapshots;

- (BOOL)allowSnapshots;

- (void)setSnapShotInterpolation:(NSImageInterpolation)interpolation;

- (NSImageInterpolation)snapShotInterpolation;

- (NSDictionary *)extImageAttributesDictionary;

- (void)setExtImageAttributesFromDictionary:(NSDictionary *)attrDictionary;

#ifdef USE_VI_COLOR_SYNC
- (void)setICCProflieData:(NSData *)ICCProfileData;

- (NSData *)ICCProfileData;
#endif
@end
